# -*- coding: utf-8 -*-

from commands import getoutput, getstatusoutput
import os
from subprocess import call, Popen, PIPE
import sys
import time
import Tkinter
import ttk
from tkMessageBox import askyesno, showerror, showinfo, showwarning

from BaseDialog import BaseDialog, msgbox_showinfo, msgbox_showerror
import ConfigHandler
from Icon import Icon
import libmisc
from popup import PopupNotification
from preferences import PreferencesDialog
from ScrolledWidgets import ScrolledListbox, ScrolledText
import TkTray
from UnicodeHandler import fsdecode, fsencode

#############################################################################

VERSION = u'1.5'
ABOUT = u'''%s
Version: %s
(c) 2015-2017 Michael Lange <klappnase@tuxfamily.org>
License: GPL v.2''' % (_('A simple software update notification applet.'), VERSION)

UPGRADESCRIPT = 'un-applet-dist-upgrade'
UPDATESCRIPT = 'un-applet-update-pkgdatabase'
SYSTEMSETTINGS = 'un-applet-system-settings'
MANAGEHOLDS = 'un-applet-manage-holds'

STATEFILE = '/var/lib/un-applet/state'
PKGSFILE = '/var/lib/un-applet/pkgnames'
LASTRUNFILE = '/var/lib/un-applet/lastrun'
READMEFILE = '/usr/share/doc/un-applet/README'

def lastrun():
    try:
        f = open(LASTRUNFILE, 'r')
        n = int(f.read().strip())
        f.close()
        return getoutput('date -d "@%d"' % n).strip()
    except (IOError, OSError, ValueError):
        return None

def lastrun_msg(msgtype='query'):
    lr = lastrun()
    if not lr:
        lr = 'N/A'
    if msgtype == 'query':
        msg = _('last query') + u': ' + fsdecode(lr)
    else:
        msg = _('last successful attempt') + u': ' + fsdecode(lr)
    return msg

#############################################################################

class PkgListViewer(BaseDialog):
    def __init__(self, master=None, command=None, **kw):

        self.command = command
        BaseDialog.__init__(self, master, icon='dialog_info',
                buttons=(_('OK'), _('Cancel')),
                message=_('Updates available'),
                detail=_('Updates for the following packages are available:'),
                **kw)

        self.img_info = Icon('dialog_info')
        self.img_error = Icon('dialog_error')
        self.info_mode = 'versions'
        # store version-info and changelogs in a dict whose elements look like:
        # 'pkgname':[version-info, changelog]
        self.pkginfo = {}

        pw = Tkinter.PanedWindow(self.frame, orient='vertical', opaqueresize=1,
                    sashwidth=3, sashrelief='ridge', showhandle=1)
        pw.pack(fill='both', expand=1, padx=10, pady=10)
        f1 = Tkinter.Frame(pw)
        f2 = Tkinter.Frame(pw)
        pw.add(f1, stretch='always')
        pw.add(f2, stretch='always')

        self.slistbox = ScrolledListbox(f1, width=100, height=200)
        self.slistbox.pack(fill='both', expand=1, pady=(0,10))
        self.listbox = self.slistbox.listbox
        self.listbox.configure(selectmode='single', exportselection=0)

        self.infolabel = Tkinter.Label(f2, justify='left',
                text=_('Version information for package "%s":') % u'N/A' + u'\n')
        self.infolabel.pack(side='top', padx=5, pady=(8,6), anchor='w')

        self.infobox = ScrolledText(f2, width=90, height=12, padx=6, pady=6,
                                     wrap='word')
        self.infobox.pack(side='bottom', fill='both', expand=1)

        self.buttons()[0].configure(command=self.on_ok)
        self.buttons()[1].configure(command=self.hide)
        self.assign_hotkeys()
        ttk.Sizegrip(self).grid(row=1, column=1, sticky='se')
        self.listbox.bind('<<ListboxSelect>>', self.show_versions)
        #self.listbox.bind('<Double-Button-1>', self.show_changelog)
        self.bind('<F2>', self.toggle_info)

    def hide(self, event=None):
        self.pkginfo = {}
        BaseDialog.hide(self)

    def on_ok(self):
        self.hide()
        if self.command:
            self.command()

    def show(self):
        self.update_pkglist()
        BaseDialog.show(self)
        self.buttons()[0].focus_set()

    def show_changelog(self, event=None):
        sel = self.listbox.curselection()
        if sel:
            self.info_mode = 'changelog'
            err_msg = u''
            pkgname = self.listbox.get(sel[0])
            if self.pkginfo[pkgname][1]:
                out = self.pkginfo[pkgname][1]
            else:
                libmisc.tk_busy(self)
                # try to determine the upgrade candidate's version
                sts, info = getstatusoutput('LC_ALL=C apt-cache policy %s' % fsencode(pkgname))
                v = None
                if not sts:
                    try:
                        v = [line.split()[1] for line in info.splitlines()
                                if line.strip().startswith('Candidate')][0]
                    except(IndexError):
                        err_msg = _('Error') + u': ' +\
                            _('could not determine the version of the upgrade candidate.') + u'\n' +\
                            _('Loading default changelog version instead.') + u'\n\n'
                if v:
                    sts, out, err = libmisc.getstatusoutput((
                        'apt-get', '-qq', 'changelog', '%s=%s' %(pkgname, v)))
                else:
                    sts, out, err = libmisc.getstatusoutput((
                        'apt-get', '-qq', 'changelog', pkgname))
                if sts:
                    out = _('Error') + u': ' + _('changelog download failed.')
                else:
                    self.pkginfo[pkgname][1] = err_msg + fsdecode(out)
                libmisc.tk_busy_forget(self)
            self.infobox.configure(state='normal')
            self.infobox.delete(1.0, 'end')
            if err_msg:
                self.infobox.insert('end', err_msg)
            self.infobox.insert('end', out)
            self.infobox.configure(state='disabled')
            self.infolabel.configure(text=_('Changelog of package "%s":') % fsdecode(pkgname) +\
                    u'\n(' + _('<F2> : show version info') + u')')

    def show_versions(self, event=None):
        sel = self.listbox.curselection()
        self.info_mode = 'versions'
        if sel:
            self.infobox.configure(state='normal')
            self.infobox.delete(1.0, 'end')
            pkgname = self.listbox.get(sel[0])
            self.infolabel.configure(
                text=_('Version information for package "%s":') % fsdecode(pkgname) +\
                u'\n(' + _('<F2> : show changelog') + u')')
            if self.pkginfo[pkgname][0]:
                info = self.pkginfo[pkgname][0]
            else:
                libmisc.tk_busy(self, cursor='')
                sts, info = getstatusoutput('apt-cache policy %s' % fsencode(pkgname))
                if sts or (not info):
                    info = 'N/A'
                else:
                    self.pkginfo[pkgname][0] = info
                libmisc.tk_busy_forget(self)
            self.infobox.insert('end', info)
            self.infobox.configure(state='disabled')
        else:
            self.infobox.configure(state='normal')
            self.infobox.delete(1.0, 'end')
            self.infobox.insert('end', 'N/A')
            self.infobox.configure(state='disabled')
            self.infolabel.configure(
                text=_('Version information for package "%s":') % u'N/A' + u'\n')

    def toggle_info(self, event):
        if self.info_mode == 'versions':
            self.info_mode = 'changelog'
            self.show_changelog()
        else:
            self.info_mode = 'versions'
            self.show_versions()

    def update_pkglist(self):
        self.listbox.select_clear(0, 'end')
        self.listbox.delete(0, 'end')
        try:
            f = open(PKGSFILE, 'r')
            lines = f.readlines()
            f.close()
            pkgs = [x.strip() for x in lines if x.strip()]
            self.icon.configure(image=self.img_info)
            self.message.configure(
                text=_('Install pending update(s) now?'))
            detail = _('Updates for the following %d package(s) are available') % len(pkgs)+\
                     u'\n(' + lastrun_msg() + u'):'
            self.detail.configure(text=detail)

            for p in pkgs:
                self.listbox.insert('end', p.strip())
                self.pkginfo[p] = [None, None]
            if pkgs:
                self.listbox.select_set(0)
                self.show_versions()
        except (IOError, OSError):
            pkgs = []
            self.icon.configure(image=self.img_error)
            self.message.configure(text=_('Error'))
            self.detail.configure(
                text=_('Error while trying to read %s') % fsdecode(PKGSFILE))

#############################################################################

class UpdateNotifierApplet(TkTray.Icon):
    def __init__(self, master, **kw):
        self.img = Icon('updates-ok')
        TkTray.Icon.__init__(self, master, image=self.img,
                             tooltip=_('Software Update'), docked=0, **kw)
        # hack for Icewm system tray:
        self.update_idletasks()
        self.configure(docked=1)
        self.update_idletasks()

        self._id = None
        # self._state holds the update procedure's current state:
        # 0 : up-to-date , > 0 : update(s) available , < 0 : error:
        # -1 : mirror(s) not reachable , -2 : pkg-database was locked
        # -3 : upgrade failed / pkgdatabase broken
        self._state = 0
        self._pipe = None
        self._poll_lock = 0
        self._backround_proc = 0 # 1=update 2=upgrade 3=system-settings 4=manage-holds
        self._popup_state = (0, 0) # (last-pkgs-state, time-of-last-popup)

        self.preferences_dialog = PreferencesDialog(trayicon=self.img)
        self.pkglist_viewer = PkgListViewer(command=self.upgrade)

        self.menu.add_command(label=_('Start upgrade...'),
                                command=self.upgrade)
        self.menu.add_command(label=_('Update package database...'),
                                command=self.update_pkg_database)
        self.menu.add_command(label=_('System-wide settings...'),
                                command=self.system_settings)
        self.menu.add_command(label=_('Manage held back packages...'),
                                command=self.manage_holds)
        self.menu.add_command(label=_('Show available upgrades...'),
                                command=self.show_state)
        self.menu.add_separator()
        self.menu.add_command(label=_('Preferences...'),
                                command=self.preferences_dialog.show)
        self.menu.add_command(label=_('About...'),
                                command=self.show_about)
        self.menu.add_separator()
        self.menu.add_command(label=_('Quit'), command=self.close)
        self.bind('<1>', self.show_state)

        if not os.path.exists(STATEFILE):
            self.firstrun()
        else:
            self.poll_loop()

    def firstrun(self):
        msg = _('Maybe you are running the applet for the first time?') + u'\n'
        msg += _('Do you want to search for pending updates now?')
        res = askyesno(title='Update Notifier Applet',
                            message=_('Status file not found'),
                            detail=msg, default='yes')
        if res:
            self.update_pkg_database(firstrun=True)
        else:
            self.poll_loop()

    def get_state(self):
        try:
            f = open(STATEFILE, 'r')
            n = int(f.read().strip())
            f.close()
            return n
        except (IOError, OSError, ValueError):
            return -256

    def poll(self):
        while self._poll_lock:
            pass
        self._poll_lock = 1
        self._state = self.get_state()
        if self._state > 0:
            newimg = 'updates-available'
            newtip = _('%d updated software package(s) are available') % self._state
        elif self._state < 0:
            newimg = 'updates-error'
            newtip = _('Error while searching for software updates')
        else:
            newimg = 'updates-ok'
            newtip = _('No software updates available')
        if newimg != self.img.image():
            self.img.image(newimg)
        self.tooltip.configure(text=newtip)

        if self._state and ConfigHandler.get('SHOW_POPUPS'):
            show_popup = 1
            last_state, last_time = self._popup_state
            now = time.time()
            if last_state == self._state:
                # only show popup if requested time interval is exceeded
                if ((now - last_time) / 60) < ConfigHandler.get('POPUP_INTERVAL'):
                    show_popup = 0
            if show_popup:
                self._popup_state = (self._state, now)
                img = 'dialog_info'
                msg = _('Software updates available')
                if self._state < 0:
                    img = 'dialog_error'
                    msg = _('An error occurred')

                def popup_balloon():
                    PopupNotification(self, icon=img, message=msg, detail=newtip+u'.',
                                duration=1000*ConfigHandler.get('POPUP_DURATION'),
                                animate=ConfigHandler.get('POPUP_ANIMATE'),
                                fixedpos=ConfigHandler.get('POPUP_FIXED_POS'))
                # add delay to give the applet enough time at wm startup
                self.after(2000, popup_balloon)
        self._poll_lock = 0

    def poll_loop(self):
        self.poll()
        self._id = self.after(60000*ConfigHandler.get('POLL_INTERVAL'), self.poll_loop)

    def poll_upgrade(self):
        # we have to call at least self._pipe.poll(), otherwise we end up
        # with a zombie process after the terminal is closed
        p = self._pipe.poll()
        if not p is None:
            pipe = self._pipe
            del(pipe)
            self._pipe = None
            self._background_proc = 0
            self.poll_loop()
            return
        self.after(500, self.poll_upgrade)

    ###########################################################################

    def manage_holds(self):
        if not self._pipe is None:
            return self.show_info_background_process()
        self._background_proc = 4
        gksu_cmd = ConfigHandler.get('GKSU_CMD').split()
        args = gksu_cmd + [MANAGEHOLDS]
        self._pipe = Popen(args, stdout=PIPE, stderr=PIPE, shell=False)
        if not self._id is None:
            _id = self._id
            self._id = None
            self.after_cancel(_id)
        self.after(500, self.poll_upgrade)

    def system_settings(self):
        if not self._pipe is None:
            return self.show_info_background_process()
        self._background_proc = 3
        gksu_cmd = ConfigHandler.get('GKSU_CMD').split()
        args = gksu_cmd + [SYSTEMSETTINGS]
        self._pipe = Popen(args, stdout=PIPE, stderr=PIPE, shell=False)
        if not self._id is None:
            _id = self._id
            self._id = None
            self.after_cancel(_id)
        self.after(500, self.poll_upgrade)

    def update_pkg_database(self, firstrun=False):
        if firstrun:
            res = True
        else:
            msg = _('Do you want to search for pending updates now?')
            res = askyesno(title='Update Notifier Applet',
                            message=_('Please confirm'),
                            detail=msg, default='yes')
        if res:
            if not self._pipe is None:
                return self.show_info_background_process()

            term_cmd = self.terminal_cmd()
            if not term_cmd:
                return libmisc.show_info_terminal_failure()

            self._background_proc = 1
            gksu_cmd = ConfigHandler.get('GKSU_CMD').split()
            args = gksu_cmd + ['%s -e %s -i' % (term_cmd, UPDATESCRIPT)]
            self._pipe = Popen(args, stdout=PIPE, stderr=PIPE, shell=False)
            if not self._id is None:
                _id = self._id
                self._id = None
                self.after_cancel(_id)
            self.after(500, self.poll_upgrade)

    def upgrade(self):
        if self._state == 0:
            msg = _('There do not seem to be any updates available.') + u'\n' +\
                  _('Do you want to try to install pending updates anyway?')
            if not askyesno(title='Update Notifier Applet', message=_('Really?'),
                            detail=msg, default='no'):
                return
        elif self._state == -1:
            return self.show_info_mirror_failure()
        elif self._state == -2:
            return self.show_info_lock_failure()
        #elif self._state == -3:
        #    return self.show_info_upgrade_failure()
        elif self._state <= -4:
            return self.show_info_statefile_failure()

        if not self._pipe is None:
            return self.show_info_background_process()

        term_cmd = self.terminal_cmd()
        if not term_cmd:
            return libmisc.show_info_terminal_failure()

        self._background_proc = 2
        # gksu might be configured with arguments, e.g. -w to
        # force su-mode, so use split() on the GKSU_CMD.
        # The command line must be passed as a single string to
        # Popen, otherwise gksu will fail to work
        gksu_cmd = ConfigHandler.get('GKSU_CMD').split()
        args = gksu_cmd + ['%s -e %s' % (term_cmd, UPGRADESCRIPT)]
        self._pipe = Popen(args, stdout=PIPE, stderr=PIPE, shell=False)
        if not self._id is None:
            _id = self._id
            self._id = None
            self.after_cancel(_id)
        self.after(500, self.poll_upgrade)

    ###########################################################################

    def show_about(self):
        try:
            f = open(READMEFILE, 'r')
            msg = f.read()
            f.close()
        except (IOError, OSError):
            msg = _('Error while trying to open "%s".') % READMEFILE
        msgbox_showinfo(message=_('About UN-Applet'), detail=ABOUT, text=msg)

    def show_info_background_process(self):
        bp = u'.\n'
        if self._background_proc == 1:
            bp = u'\n(%s).\n' % _('update of the package database')
        elif self._background_proc == 2:
            bp = u'\n(%s).\n' % _('software upgrade')
        elif self._background_proc == 3:
            bp = u'\n(%s).\n' % _('un-applet System Settings')
        elif self._background_proc == 4:
            bp = u'\n(%s).\n' % _('manage held back packages')
        msg = _('It seems like a background process is already running') +\
                  bp + _('Please try again later.')
        showerror(title='Update Notifier Applet', message=_('Error'),
                         detail=msg)

    def show_info_lock_failure(self):
        msg = _('Error while trying to get lock /var/lib/dpkg/lock') + u' ' +\
              _('or /var/lib/apt/lists/lock.') +\
              u'\n(' + lastrun_msg(msgtype='attempt') + u')' + u'\n\n' +\
              _('Possible reason:') + u'\n' + \
              _('The package database was locked by another process') + u' ' +\
              _('while trying to check for pending updates.') + u'\n\n' +\
              _('Possible solutions:') + u'\n\n' +\
              _('Make sure that no other package management application is running') + u', ' +\
              _('then run "Update package database..." from the menu') + u' ' +\
              _('or simply wait until the next automatic query for pending updates is performed.')

        msgbox_showerror(message=_('Error'),
                    detail=_('Please note the error details below:'), text=msg)

    def show_info_mirror_failure(self):
        msg = _('Error while trying to download package data files.') + u'\n' +\
              _('One or more mirror servers could not be reached.') +\
              u'\n(' + lastrun_msg(msgtype='attempt') + u')' + u'\n\n' +\
              _('Possible reasons:') + u'\n' + \
              _('1. your network connection might be down or unreliable') + u'\n' +\
              _('2. one or more of the mirrors defined in %s or %s ') % (u'/etc/apt/sources.list', u'/etc/apt/sources.list.d/*.list') +\
              _('is temporarily or permanently not working.') + u'\n\n' +\
              _('In case 2. you can try to update the package database at a later time') + u' ' +\
              _('or, if the problem persists, remove the broken mirror server(s) from the sources.list file(s).') + u'\n\n' + \
              _('If you want to skip the test of the mirror servers') + u' ' +\
              _('before updating the package database, you may open the file /etc/un-applet/un-applet.conf') + u' ' +\
              _('and set VERIFY_MIRRORS="0" .') + u'\n' +\
              _('However, due to a bug in apt-get this may cause packages erroneously selected for upgrade') + u' ' +\
              _('if one or more mirrors are unreachable, especially if you use apt-pinning') + u' ' +\
              _('which may very well be the case even if you never consciously decided to do so.') + u'\n\n' +\
              _('You can obtain more detailed information about which mirrors are not reachable') + u' ' +\
              _('by running "Update package database..." from the context menu and examining the output messages.') + u' ' +\
              _('It is possible that this will even fix the problem') + u' ' +\
              _('in case that e.g. a temporary network connection issue has been resolved.')

        msgbox_showerror(message=_('Error'),
                    detail=_('Please note the error details below:'), text=msg)

    def show_info_statefile_failure(self):
        msg = _('Error while trying to analyse the file /var/lib/un-applet/state') +\
              u'\n(' + lastrun_msg(msgtype='attempt') + u')' + u'\n\n' +\
              _('Possible reasons:') + u'\n' + \
              _('1. the file has not yet been created because') + u' ' +\
              _('you are running the Update Notifier Applet for the first time.') + u'\n' +\
              _('2. the file was removed or modified by another process.') + u'\n\n' +\
              _('Possible solutions:') + u'\n\n' +\
              _('You can run "Update package database..." from the menu') + u' ' +\
              _('or simply wait until the next automatic query for pending updates is performed.')

        msgbox_showerror(message=_('Error'),
                    detail=_('Please note the error details below:'), text=msg)

    def show_info_upgrade_failure(self):
        msg = _('An error occurred during the upgrade procedure.') + u' ' +\
              _("Your system's package database might be corrupted.") + u'\n\n' +\
              _('Possible reasons:') + u'\n' + \
              _('There may be unresolved dependencies or a broken package has been installed.') + u'\n\n' +\
              _('Possible solutions:') + u'\n\n' +\
              _('Select "Start upgrade..."') + u' ' +\
              _("from un-applet's menu or") + u' ' +\
              _('open a terminal window and run (as root) the command') + u' ' +\
              _('you prefer to upgrade your system.') + u'\n' + \
              _('Read thoroughly through any error messages the command returns') + u' ' +\
              _('and the suggestions it makes to solve the problem.') + u' ' +\
              _('If there are unresolved dependencies it may help') + u' ' +\
              _('to manually install one or more missing packages.')

        msgbox_showerror(message=_('Error'),
                    detail=_('Please note the error details below:'), text=msg)

    def show_info_uptodate(self):
        msg = _('No updates available, your system is up to date.') + \
                        u'\n(' + lastrun_msg() + u')'
        showinfo(title='Update Notifier Applet', message=_('Information'),
                        detail=msg)

    def show_state(self, event=None):
        self.poll()
        if self._state == 0:
            self.show_info_uptodate()
        elif self._state > 0:
            self.pkglist_viewer.show()
        elif self._state == -1:
            self.show_info_mirror_failure()
        elif self._state == -2:
            self.show_info_lock_failure()
        elif self._state == -3:
            self.show_info_upgrade_failure()
        elif self._state <= -4:
            self.show_info_statefile_failure()

    def terminal_cmd(self):
        terminal = libmisc.get_terminal_cmd(preferred=ConfigHandler.get('PREF_TERM'))
        if not terminal:
            return None

        term_cmd = '%s -title SoftwareUpdate -sb ' % terminal
        if os.path.isfile(os.path.join(libmisc.ICONDIR, 'un-applet.xpm')):
            if os.path.basename(terminal) == 'urxvt':
                term_cmd += '-icon %s ' % os.path.join(libmisc.ICONDIR, 'un-applet.xpm')
            elif os.path.basename(terminal) == 'mlterm':
                term_cmd += '--iconpath %s ' % os.path.join(libmisc.ICONDIR, 'un-applet.xpm')

        term_cmd += '-geometry %dx%d+%d+%d -fg %s -bg %s' % (
                        ConfigHandler.get('TERM_WIDTH'),
                        ConfigHandler.get('TERM_HEIGHT'),
                        ConfigHandler.get('TERM_XOFF'),
                        ConfigHandler.get('TERM_YOFF'),
                        ConfigHandler.get('TERM_FG'),
                        ConfigHandler.get('TERM_BG'))
        return term_cmd

    def close(self):
        if not self._id is None:
            _id = self._id
            self._id = None
            self.after_cancel(_id)
        self.preferences_dialog.hide()
        self.preferences_dialog.destroy()
        self.pkglist_viewer.destroy()
        self.destroy()
        self.master.quit()
