# -*- coding: utf-8 -*-

import gettext
import os
import signal
from subprocess import Popen, PIPE
import sys
import Tkinter
import tkMessageBox
import ttk

INSTALLDIR = os.path.abspath(sys.path[0])
ICONDIR = os.path.join(INSTALLDIR, 'icons')
CONFIGDIR = os.path.join(os.getenv('HOME'), '.config', 'un-applet')

def install_gettext():
    if INSTALLDIR.startswith('/usr/local'):
        localepath = '/usr/local/share/locale'
    elif INSTALLDIR.startswith('/usr'):
        localepath = '/usr/share/locale'
    else:
        localepath = os.path.join(INSTALLDIR, 'locale')
    gettext.install('un-applet', localepath, unicode=1)

######################################################################

class InvisibleRootWindow(Tkinter.Tk):
    def __init__(self, *args, **kw):
        Tkinter.Tk.__init__(self, *args, **kw)

        self._signal_id = None

        self.withdraw()
        if not self.png_support():
            self.quit()
        self.add_windowicon()
        self.option_add('*selectBackground', '#4a6984')
        self.option_add('*selectForeground', '#ffffff')
        self.option_add('*inactiveSelectBackground', '#4a6984')
        self.bind_class('TButton', '<Return>', self._on_key_return)
        self.bind_class('TCheckbutton', '<Return>', self._on_key_return)
        self.bind_class('TRadiobutton', '<Return>', self._on_key_return)
        ttk.Style().theme_use('alt')
        self.poll_signal()

    def _on_key_return(self, event):
        try:
            event.widget.event_generate('<space>')
        except(AttributeError):
            pass # may happen with tkMessageBoxes

    def add_windowicon(self):
        try:
            self.iconimg = Tkinter.PhotoImage(
                                file=os.path.join(ICONDIR, 'un-applet.png'))
            self.tk.call('wm', 'iconphoto', '.', '-default', self.iconimg)
        except:
            pass

    def destroy(self):
        if not self._signal_id is None:
            _id = self._signal_id
            self._signal_id = None
            self.after_cancel(_id)
        Tkinter.Tk.destroy(self)

    def png_support(self):
        if Tkinter.TkVersion < 8.6:
            try:
                tkimgver = root.tk.call('package', 'require', 'img::png')
            except(Tkinter.TclError):
                msg = _('No PNG image support available, aborting.') + u'\n' +\
                      _('Please install the tk image library') +\
                       u'\n(apt-get install libtk-img).'
                tkMessageBox.showerror(title='Update Notifier Applet',
                            message=_('Error'), detail=msg)
                return False
        return True

    def poll_signal(self):
        self._signal_id = self.after(500, self.poll_signal)

    def quit_command(self, cmd):
        self.protocol('WM_DELETE_WINDOW', cmd)
        for signum in (2, 3, 6, 15):
            signal.signal(signum, cmd)

def mainwindow():
    Tkinter.wantobjects = 0
    root = InvisibleRootWindow()
    return root

######################################################################

def getstatusoutput(args, nice=True):
    args = list(args)
    if nice:
        args = ['nice', '-n', '19'] + args
    pipe = Popen(args, stdout=PIPE, stderr=PIPE, bufsize=-1)
    output, errors = pipe.communicate()
    status = pipe.returncode
    del(pipe)
    return status, output, errors

# used in both applet.py and preferences.py :

def get_terminal_cmd(preferred=None):
    terminal = None
    alternatives = ['mlterm', 'urxvt', 'xvt']
    if preferred:
        alternatives = [preferred] + alternatives
    for a in alternatives:
        terminal = which(a)
        if terminal:
            break
    return terminal

def show_info_terminal_failure():
    msg = _('No supported terminal emulator application found.') + u'\n' +\
          _('Please make sure that one of the programs') +\
          u' "xvt", "rxvt-unicode" ' +  _('or') + u' "mlterm" ' +\
          _('is installed on your system.')
    showerror(title='Update Notifier Applet', message=_('Error'),
                detail=msg)

def which(program):
    if os.path.exists(program):
        full = program
    else:
        full = ''
        paths = os.getenv('PATH').split(os.pathsep)
        for p in paths:
            full = os.path.join(p, program)
            if os.path.exists(full):
                break
    if os.path.isfile(full) and os.access(full, os.X_OK):
        return full
    return None

# missing from Tkinter.py:

def grid_anchor(widget, anchor):
    widget.tk.call('grid', 'anchor', widget._w, anchor)

def tk_busy(window, **kw):
    try:
        window.tk.call(('tk', 'busy', 'hold', window._w) + window._options(kw))
        window.update()
    except(Tkinter.TclError):
        # Tk-8.5 compatibility mode
        pass

def tk_busy_forget(window):
    try:
        window.tk.call('tk', 'busy', 'forget', window._w)
    except(Tkinter.TclError):
        pass

# ttk.Combobox widget in readonly state with a focus highlight
class ReadonlyCombobox(ttk.Combobox):
    # idea borrowed from ScrolledText
    def __init__(self, master=None, **kw):
        self.frame = Tkinter.Frame(master, highlightthickness=1)
        s = ttk.Style()
        s.configure('ROCombo.TCombobox')
        # get rid of the gray background and the useless entry selection
        s.map('ROCombo.TCombobox', fieldbackground=[('readonly','#ffffff')])
        s.map('ROCombo.TCombobox', selectbackground=[('readonly', '#ffffff')])
        s.map('ROCombo.TCombobox', selectforeground=[('readonly', '#000000')])
        ttk.Combobox.__init__(self, self.frame, style='ROCombo.TCombobox',
                                state='readonly', **kw)
        self.pack(side='left', fill='both', expand=1)
        text_meths = vars(ttk.Combobox).keys()
        methods = vars(Tkinter.Pack).keys() + vars(Tkinter.Grid).keys() + \
                  vars(Tkinter.Place).keys()
        methods = set(methods).difference(text_meths)
        for m in methods:
            if m[0] != '_' and m != 'config' and m != 'configure':
                setattr(self, m, getattr(self.frame, m))

    def __str__(self):
        return str(self.frame)

