# -*- coding: utf-8 -*-

import os
from subprocess import Popen
import Tkinter
import ttk
from tkMessageBox import showerror

from BaseDialog import BaseDialog
import ConfigHandler
from Counter import Counter
from Icon import Icon
import libmisc
from UnicodeHandler import fsdecode, fsencode

# a hopefully sensible selection of colors from rgb.txt which should
# hopefully work with any terminal
RGB_TXT = '''White Black Blue1 Blue2 Blue3 Blue4 Cyan1 Cyan2 Cyan3 Cyan4
 Green1 Green2 Green3 Green4 Yellow1 Yellow2 Yellow3 Yellow4 Orange1 Orange2
 Orange3 Orange4 Red1 Red2 Red3 Red4 Pink1 Pink2 Pink3 Pink4 Magenta1 Magenta2
 Magenta3 Magenta4 Purple1 Purple2 Purple3 Purple4 Tan1 Tan2 Tan3 Tan4 Brown1
 Brown2 Brown3 Brown4 Lightgreen Lightblue Lightcyan Lightyellow Darkblue
 Darkcyan Darkmagenta Darkred AntiqueWhite LightGoldenrod MistyRose
 PaleTurquoise PowderBlue DarkOliveGreen DarkSlateGray MidnightBlue Firebrick
 DarkSlateBlue Gray10 Gray20 Gray30 Gray40 Gray50 Gray60
 Gray70 Gray80 Gray90'''
COLORS = [c.strip() for c in RGB_TXT.split()]

###########################################################################

class PreferencesDialog(BaseDialog):
    def __init__(self, master=None, trayicon=None, **kw):
        BaseDialog.__init__(self, master, icon='preferences',
                        message=_('Preferences'),
                        detail=_('Customize the behavior of un-applet'),
                        buttons=(_('Apply'), _('Close')), **kw)
        self._pipe = None
        self.trayicon = trayicon# reference for switching icon on theme change

        self.pref_term = Tkinter.StringVar(value=ConfigHandler.get('PREF_TERM'))
        self.term_bg = Tkinter.StringVar(value=ConfigHandler.get('TERM_BG'))
        self.term_fg = Tkinter.StringVar(value=ConfigHandler.get('TERM_FG'))
        self.gksu_cmd = Tkinter.StringVar(value=ConfigHandler.get('GKSU_CMD'))
        self.icon_theme = Tkinter.StringVar(value=ConfigHandler.get('ICON_THEME'))
        self.show_popups = Tkinter.IntVar(value=ConfigHandler.get('SHOW_POPUPS'))
        self.popup_animate = Tkinter.IntVar(value=ConfigHandler.get('POPUP_ANIMATE'))
        self.popup_fixed_pos = Tkinter.IntVar(value=ConfigHandler.get('POPUP_FIXED_POS'))

        self.img_err = Icon('error')

        self.frame.configure(relief='groove', bd=2)
        self.frame.grid_configure(padx=5, pady=5)
        self.frame.grid_columnconfigure(1, weight=1)

        Tkinter.Label(self.frame, text=_('Time interval after which to poll for pending updates:'), anchor='w').grid(
                        row=2, column=0, padx=(10,0), pady=(15,10), sticky='w')
        self.poll_int = Counter(self.frame,
            rlabel=_('minutes'), value=ConfigHandler.get('POLL_INTERVAL'),
            width=6, from_=3, to=9999)
        self.poll_int.grid(row=2, column=1, padx=10, pady=(15,10), sticky='w')

        Tkinter.Label(self.frame, text=_('Startup delay (for IceWM system tray):'), anchor='w').grid(
                        row=3, column=0, padx=(10,0), pady=(0,10), sticky='w')
        self.startup_delay = Counter(self.frame,
            rlabel=_('seconds'), value=ConfigHandler.get('STARTUP_DELAY'),
            width=6, from_=0, to=20)
        self.startup_delay.grid(row=3, column=1, padx=10, pady=(0,10), sticky='w')

        Tkinter.Label(self.frame, text=_('Graphical administrator login command:'), anchor='w').grid(
                        row=4, column=0, padx=(10,0), pady=(0,10), sticky='w')
        self.e2 = Tkinter.Entry(self.frame, textvariable=self.gksu_cmd)
        self.e2.grid(row=4, column=1, padx=10, pady=(0,10), sticky='we')

        Tkinter.Label(self.frame, text=_('Icon theme:'), anchor='w').grid(
                        row=5, column=0, padx=(10,0), pady=(0,10), sticky='w')
        self.cb0 = libmisc.ReadonlyCombobox(self.frame, values=(), textvariable=self.icon_theme)
        self.cb0.grid(row=5, column=1, padx=10, pady=(0, 10), sticky='w')
        self.cb0.bind('<<ComboboxSelected>>', self.preview_icontheme)

        f0 = Tkinter.Frame(self.frame)
        f0.grid(row=6, column=0, columnspan=2, padx=(10,0), sticky='ew')
        ttk.Checkbutton(f0, text=_('Show notification balloons'),
                        variable=self.show_popups, command=self.toggle_popups,
                        onvalue=1, offvalue=0).grid(row=0, column=0,
                        columnspan=2, pady=(0,5), sticky='w')
        self.popup_duration = Counter(f0,
                        label=_('Display duration:'), rlabel=_('sec.'),
                        value=ConfigHandler.get('POPUP_DURATION'),
                        width=6, from_=1, to=20)
        self.popup_duration.grid(row=1, column=0, padx=15, pady=(0,5), sticky='w')
        self.popup_interval = Counter(f0, label=_('Repeat after:'),
                        rlabel=_('min.') + u' (' + _('0 = never') + u')',
                        value=ConfigHandler.get('POPUP_INTERVAL'),
                        width=6, from_=0, to=300, increment=10)
        self.popup_interval.grid(row=1, column=1, padx=15, pady=(0,5), sticky='w')
        self.check0 = ttk.Checkbutton(f0, text=_('Animate notification balloons'),
                        variable=self.popup_animate,
                        onvalue=1, offvalue=0)
        self.check0.grid(row=2, column=0, padx=15, pady=(0,10), sticky='w')
        self.check1 = ttk.Checkbutton(f0, text=_('Use fixed screen position'),
                        variable=self.popup_fixed_pos,
                        onvalue=1, offvalue=0)
        self.check1.grid(row=2, column=1, padx=15, pady=(0,10), sticky='w')

        lf = Tkinter.LabelFrame(self.frame, text=_('Terminal emulator options:'))
        lf.grid(row=8, column=0, columnspan=2, padx=3, pady=(0,3), sticky='nsew')
        lf.grid_columnconfigure(2, weight=1)

        f1 = Tkinter.Frame(lf)
        f1.grid(row=0, column=0, columnspan=2, sticky='ew')
        Tkinter.Label(f1, text=_('Preferred terminal application (if available):')).grid(
                        row=0, column=0, columnspan=3, sticky='w', padx=5, pady=(5,0))
        self.rb1 = ttk.Radiobutton(f1, text='Xvt', value='xvt', variable=self.pref_term)
        self.rb1.grid(row=1, column=0, padx=10, pady=5, sticky='w')
        self.rb2 = ttk.Radiobutton(f1, text='Rxvt-Unicode', value='urxvt', variable=self.pref_term)
        self.rb2.grid(row=1, column=1, padx=10, pady=5, sticky='w')
        self.rb3 = ttk.Radiobutton(f1, text='MLTerm', value='mlterm', variable=self.pref_term)
        self.rb3.grid(row=1, column=2, padx=10, pady=5, sticky='w')

        Tkinter.Label(lf, text=_('Background color:')).grid(
                                row=1, column=0, padx=5, pady=5, sticky='w')
        self.cb1 = libmisc.ReadonlyCombobox(lf, values=COLORS, textvariable=self.term_bg)
        self.cb1.grid(row=1, column=1, padx=5, pady=5, sticky='w')
        Tkinter.Label(lf, text=_('Text color:')).grid(
                                row=2, column=0, padx=5, pady=5, sticky='w')
        self.cb2 = libmisc.ReadonlyCombobox(lf, values=COLORS, textvariable=self.term_fg)
        self.cb2.grid(row=2, column=1, padx=5, pady=5, sticky='w')
        self.previewtext = 'Spam, Spam, Spam, Spam...\n' +\
                           'Lovely Spam! Wonderful Spam!'
        self.preview = Tkinter.Label(lf, relief='sunken', bd=2, image='',
            compound='left', padx=5, font='TkFixedFont', justify='left',
            anchor='w', text=self.previewtext)
        try:
            self.preview.configure(fg=self.term_fg.get(), bg=self.term_bg.get())
        except Tkinter.TclError:
            # just in case...
            self.term_fg.set('Black')
            self.term_bg.set('White')
            self.preview.configure(fg=self.term_fg.get(), bg=self.term_bg.get())
        self.preview.grid(row=1, column=2, rowspan=2, pady=5, padx=5, sticky='nsew')
        self.term_fg.trace('w', self.update_preview_fg)
        self.term_bg.trace('w', self.update_preview_bg)
        Tkinter.Label(lf, text=_('Please note:') + u' ' +\
                _('colors might be reversed on some systems/terminals.')).grid(
                row=3, column=0, columnspan=3, padx=5, pady=5, sticky='w')

        f1 = Tkinter.Frame(lf)
        f1.grid(row=4, column=0, columnspan=3, sticky='w')
        f1.grid_columnconfigure(0, weight=1)

        Tkinter.Label(f1, text=_('Window size (in characters):')).grid(
            row=0, column=0, padx=5, pady=5, sticky='w')
        self.term_width = Counter(f1, label=_('Width:'),
            value=ConfigHandler.get('TERM_WIDTH'),
            width=6, from_=40, to=400)
        self.term_width.grid(row=0, column=1, padx=5, pady=5, sticky='e')
        self.term_height = Counter(f1, label=_('Height:'),
            value=ConfigHandler.get('TERM_HEIGHT'),
            width=6, from_=10, to=150)
        self.term_height.grid(row=0, column=2, padx=5, pady=5, sticky='e')

        Tkinter.Label(f1, text=_('Window position (in pixels):')).grid(
            row=1, column=0, padx=5, pady=5, sticky='w')
        self.term_xoff = Counter(f1, label=_('Horizontal pos.:'),
            value=ConfigHandler.get('TERM_XOFF'),
            width=6, from_=0, to=1000)
        self.term_xoff.grid(row=1, column=1, padx=5, pady=5, sticky='e')
        self.term_yoff = Counter(f1, label=_('Vertical pos.:'),
            value=ConfigHandler.get('TERM_YOFF'),
            width=6, from_=0, to=1000)
        self.term_yoff.grid(row=1, column=2, padx=5, pady=5, sticky='e')

        ttk.Button(lf, text=_('Test terminal configuration...'),
                    command=self.test_terminal).grid(
                    row=5, column=0, padx=5, pady=5, sticky='w')

        self.buttons()[0].configure(command=self.apply, state='disabled')
        self.buttons()[1].configure(command=self.hide)
        self.assign_hotkeys()

        for var in (self.poll_int, self.startup_delay, self.gksu_cmd,
                    self.icon_theme, self.pref_term, self.term_bg,
                    self.term_fg, self.term_width, self.term_height,
                    self.term_xoff, self.term_yoff, self.show_popups,
                    self.popup_duration, self.popup_interval,
                    self.popup_animate, self.popup_fixed_pos):
            var.trace('w', self.track_changes)

    ##################################################################

    def toggle_popups(self):
        newstate = 'disabled'
        if self.show_popups.get():
            newstate = 'normal'
        self.popup_interval.state(newstate)
        self.popup_duration.state(newstate)
        self.check0.configure(state=newstate)
        self.check1.configure(state=newstate)

    def check_iconthemes(self):
        iconthemes = ['Default']
        dirnames = [os.path.join(libmisc.ICONDIR, d) for d in os.listdir(libmisc.ICONDIR) \
                        if os.path.isdir(os.path.join(libmisc.ICONDIR, d))] + \
                   [os.path.join(libmisc.CONFIGDIR, d) for d in os.listdir(libmisc.CONFIGDIR) \
                        if os.path.isdir(os.path.join(libmisc.CONFIGDIR, d))]
        for d in dirnames:
            try:
                # check if directory contains at least one .png file
                for f in os.listdir(d):
                    if os.path.splitext(f)[1] == '.png':
                        iconthemes.append(os.path.basename(d))
                        break
            except:# read permission denied
                continue
        iconthemes.sort()
        self.cb0.configure(values=iconthemes)
        self.icon_theme.set(ConfigHandler.get('ICON_THEME'))

    def preview_icontheme(self, event):
        oldtheme = ConfigHandler.get('ICON_THEME')
        ConfigHandler.set('ICON_THEME', self.icon_theme.get())
        self.trayicon.image(self.trayicon.image())
        ConfigHandler.set('ICON_THEME', oldtheme)

    def check_installed_terminals(self):
        if not libmisc.which('xvt'):
            self.rb1.configure(state='disabled')
        else:
            self.rb1.configure(state='normal')
        if not libmisc.which('urxvt'):
            self.rb2.configure(state='disabled')
        else:
            self.rb2.configure(state='normal')
        if not libmisc.which('mlterm'):
            self.rb3.configure(state='disabled')
        else:
            self.rb3.configure(state='normal')

    def update_preview_bg(self, *args):
        oldbg = self.preview.cget('bg')
        try:
            self.preview.configure(bg=self.term_bg.get())
        except Tkinter.TclError:
            m = _('Color not supported') + u':\n' + fsdecode(self.term_bg.get())
            self.preview.configure(text=m, image=self.img_err,
                                    bg='white', fg='black')
            self.term_bg.set(oldbg)
            def restore_preview_bg():
                self.preview.configure(text=self.previewtext, image='',
                                        fg=self.term_fg.get(), bg=oldbg)
            self.after(2000, restore_preview_bg)

    def update_preview_fg(self, *args):
        oldfg = self.preview.cget('fg')
        try:
            self.preview.configure(fg=self.term_fg.get())
        except Tkinter.TclError:
            m = _('Color not supported') + u':\n' + fsdecode(self.term_fg.get())
            self.preview.configure(text=m, image=self.img_err,
                                    bg='white', fg='black')
            self.term_fg.set(oldfg)
            def restore_preview_fg():
                self.preview.configure(text=self.previewtext, image='',
                                        bg=self.term_bg.get(), fg=oldfg)
            self.after(2000, restore_preview_fg)

    def test_terminal(self):
        if not self._pipe is None:
            msg = _('A test window is already running.')
            return showerror(parent=self, title='Update Notifier Applet',
                             message=_('Error'), detail=msg)

        terminal = libmisc.get_terminal_cmd(preferred=self.pref_term.get())
        if not terminal:
            return libmisc.show_info_terminal_failure()
        term = '%s -title SoftwareUpdate -sb -geometry %dx%d+%d+%d -fg %s -bg %s -e' % (
                    terminal,
                    self.term_width.get(),
                    self.term_height.get(),
                    self.term_xoff.get(),
                    self.term_yoff.get(),
                    self.term_fg.get(),
                    self.term_bg.get())
        prompt1 = _('Test window for your terminal emulator configuration.')
        prompt2 = _('Please press Enter to close the window.')
        args = term.split() +\
            ['python', '-c', u'print("%s"); raw_input("%s")' % (prompt1, prompt2)]
        self._pipe = Popen(args)
        self.after(500, self.poll_test_terminal)

    def poll_test_terminal(self):
        # we have to call at least self._pipe.poll(), otherwise we end up
        # with a zombie process after the terminal is closed
        p = self._pipe.poll()
        if not p is None:
            pipe = self._pipe
            del(pipe)
            self._pipe = None
            return
        self.after(500, self.poll_test_terminal)

    ##################################################################

    def settings_changed(self):
        if self.poll_int.get(raw=True) != ConfigHandler.get('POLL_INTERVAL'):
            return True
        if self.startup_delay.get(raw=True) != ConfigHandler.get('STARTUP_DELAY'):
            return True
        if self.gksu_cmd.get() != ConfigHandler.get('GKSU_CMD'):
            return True
        if self.icon_theme.get() != ConfigHandler.get('ICON_THEME'):
            return True
        if self.pref_term.get() != ConfigHandler.get('PREF_TERM'):
            return True
        if self.term_bg.get() != ConfigHandler.get('TERM_BG'):
            return True
        if self.term_fg.get() != ConfigHandler.get('TERM_FG'):
            return True
        if self.term_width.get(raw=True) != ConfigHandler.get('TERM_WIDTH'):
            return True
        if self.term_height.get(raw=True) != ConfigHandler.get('TERM_HEIGHT'):
            return True
        if self.term_xoff.get(raw=True) != ConfigHandler.get('TERM_XOFF'):
            return True
        if self.term_yoff.get(raw=True) != ConfigHandler.get('TERM_YOFF'):
            return True
        if self.show_popups.get() != ConfigHandler.get('SHOW_POPUPS'):
            return True
        if self.popup_duration.get(raw=True) != ConfigHandler.get('POPUP_DURATION'):
            return True
        if self.popup_interval.get(raw=True) != ConfigHandler.get('POPUP_INTERVAL'):
            return True
        if self.popup_animate.get() != ConfigHandler.get('POPUP_ANIMATE'):
            return True
        if self.popup_fixed_pos.get() != ConfigHandler.get('POPUP_FIXED_POS'):
            return True
        return False

    def track_changes(self, *args):
        if self.settings_changed():
            self.buttons()[0].configure(state='normal')
        else:
            self.buttons()[0].configure(state='disabled')

    def validate(self):
        ok = True
        gksu = self.gksu_cmd.get().strip()
        self.gksu_cmd.set(gksu) # keep the entry in sync with ConfigHandler
        if gksu:
            full = libmisc.which(gksu.split()[0])
            if not full:
                ok = False
                msg = _('Program not found or permission denied:') +\
                        u'\n"%s"' % fsdecode(gksu.split()[0])
        else:
            ok = False
            msg = _('Please define a root login program to use.')
        if not ok:
            showerror(parent=self, title='Update Notifier Applet',
                        message=_('Error'), detail=msg)
            self.e2.focus_set()
            self.e2.selection_clear()
            self.e2.selection_range(0, 'end')
            return False
        for w in (self.poll_int, self.startup_delay, self.term_width,
                    self.term_height, self.term_xoff, self.term_yoff,
                    self.popup_duration, self.popup_interval):
            try:
                # don't forget to convert w.spinbox['from'/'to'] into a float
                # first, since Tk will return strings that look like floats!
                min, max, value = (int(float(w.spinbox['from'])),
                                    int(float(w.spinbox['to'])), int(w.get()))
                if not min <= value <= max:
                    ok = False
            except ValueError:
                ok = False
            if not ok:
                msg = _('Please choose an integer value between %d and %d') % (min, max)
                showerror(parent=self, title='Update Notifier Applet',
                            message=_('Error'), detail=msg)
                w.focus_set()
                w.selection('clear')
                w.selection('range', 0, 'end')
                return False
        if self.term_bg.get() == self.term_fg.get():
            msg = _('Please choose different fore- and background colors.')
            showerror(parent=self, title='Update Notifier Applet',
                        message=_('Error'), detail=msg)
            return False
        return True

    ##################################################################

    def apply(self):
        if self.settings_changed() and self.validate():
            ConfigHandler.set('POLL_INTERVAL', self.poll_int.get())
            ConfigHandler.set('STARTUP_DELAY', self.startup_delay.get())
            ConfigHandler.set('GKSU_CMD', self.gksu_cmd.get().strip())
            ConfigHandler.set('TERM_BG', self.term_bg.get())
            ConfigHandler.set('TERM_FG', self.term_fg.get())
            ConfigHandler.set('TERM_WIDTH', self.term_width.get())
            ConfigHandler.set('TERM_HEIGHT', self.term_height.get())
            ConfigHandler.set('TERM_XOFF', self.term_xoff.get())
            ConfigHandler.set('TERM_YOFF', self.term_yoff.get())
            ConfigHandler.set('PREF_TERM', self.pref_term.get())
            ConfigHandler.set('ICON_THEME', self.icon_theme.get())
            ConfigHandler.set('SHOW_POPUPS', self.show_popups.get())
            ConfigHandler.set('POPUP_DURATION', self.popup_duration.get())
            ConfigHandler.set('POPUP_INTERVAL', self.popup_interval.get())
            ConfigHandler.set('POPUP_ANIMATE', self.popup_animate.get())
            ConfigHandler.set('POPUP_FIXED_POS', self.popup_fixed_pos.get())
            ConfigHandler.save()
            self.track_changes()

    def hide(self):
        for w in (self.poll_int, self.startup_delay, self.term_width,
                    self.term_height, self.term_xoff, self.term_yoff,
                    self.popup_duration, self.popup_interval):
            w.selection('clear')
        for w in (self.e2, self.cb1, self.cb2):
            w.selection_clear()
        if not self._pipe is None:
            self._pipe.terminate()
        self.trayicon.image(self.trayicon.image())
        BaseDialog.hide(self)

    def show(self):
        self.poll_int.set(ConfigHandler.get('POLL_INTERVAL'))
        self.startup_delay.set(ConfigHandler.get('STARTUP_DELAY'))
        self.gksu_cmd.set(ConfigHandler.get('GKSU_CMD'))
        self.term_bg.set(ConfigHandler.get('TERM_BG'))
        self.term_fg.set(ConfigHandler.get('TERM_FG'))
        self.term_width.set(ConfigHandler.get('TERM_WIDTH'))
        self.term_height.set(ConfigHandler.get('TERM_HEIGHT'))
        self.term_xoff.set(ConfigHandler.get('TERM_XOFF'))
        self.term_yoff.set(ConfigHandler.get('TERM_YOFF'))
        self.pref_term.set(ConfigHandler.get('PREF_TERM'))
        self.show_popups.set(ConfigHandler.get('SHOW_POPUPS'))
        self.popup_duration.set(ConfigHandler.get('POPUP_DURATION'))
        self.popup_interval.set(ConfigHandler.get('POPUP_INTERVAL'))
        self.popup_animate.set(ConfigHandler.get('POPUP_ANIMATE'))
        self.popup_fixed_pos.set(ConfigHandler.get('POPUP_FIXED_POS'))
        self.toggle_popups()
        self.check_installed_terminals()
        self.check_iconthemes()
        BaseDialog.show(self)

