#!/usr/bin/python2
# -*- coding: utf-8 -*-
###############################################################################
# un-applet
# copyright (c) 2015-2016 Michael Lange <klappnase@tuxfamily.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the #hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

import libmisc
libmisc.install_gettext()

import ConfigHandler
ConfigHandler.initialize()

from commands import getoutput, getstatusoutput
import fnmatch
import os
import sys
from subprocess import call
import Tkinter
import ttk
from tkMessageBox import askyesno, showinfo, showerror

from BaseDialog import BaseDialog, msgbox_showinfo, msgbox_askyesno, msgbox_askokcancel
from Icon import Icon
from ScrolledWidgets import ScrolledTreeview, ScrolledText
from UnicodeHandler import fsdecode, fsencode

HAVE_APTITUDE = libmisc.which('aptitude') and True
if HAVE_APTITUDE:
    HAVE_PERMISSIONS = os.access('/var/lib/dpkg', os.W_OK) and os.access('/var/lib/aptitude', os.W_OK)
else:
    HAVE_PERMISSIONS = os.access('/var/lib/dpkg', os.W_OK)

###############################################################################

class InfoText(ScrolledText):
    def __init__(self, master=None, images=(), **kw):
        ScrolledText.__init__(self, master, **kw)
        self.configure(bg=self.frame['bg'], relief='flat', bd=0,
                       font='TkDefaultFont', padx=5, pady=5,
                       selectborderwidth=0, selectforeground='black',
                       inactiveselectbackground=self.frame['bg'])

        self.tag_configure('caption', font='TkCaptionFont', spacing3=6)
        self.tag_configure('orange', background='#FF903D')
        self.tag_configure('green', background='#2CF88A')
        self.tag_configure('pink', background='#FA89DD')

        self.insert('end', _('Manage held back packages') + u'\n', 'caption')
        self.insert('end',_('If you want to keep the specific version of certain packages') +\
                u', ' + _('you can put these packages on "hold".') + u' ' +\
                _('No updates for these packages will be installed during future software upgrades.') + u'\n\n')

        self.insert('end', _('Use the "Filter" entry below to find a particular package') + u' ')
        self.insert('end', _('in the installed packages list'))
        self.insert('end', u' ' + _('("*" and "?" wildcards are supported)') + u'.\n\n')

        self.insert('end', _("The package's checkbox shows the action") +\
                    u' ' + _('that will be performed for this package:') + u'\n')

        # images = (img_checked, img_unchecked, img_unhold)
        self.image_create('end', image=images[0], align='center')
        self.insert('end', u'   ' + _('The package will be put on hold for "apt-get" and') +\
                    u' ' + _('(if installed) "aptitude".') + u'\n')

        self.image_create('end', image=images[2], align='center')
        if HAVE_APTITUDE:
            self.insert('end', u'   ' +_('All current "hold"-mark(s) for this package will be removed.') + u'\n')
            self.image_create('end', image=images[1], align='center')
            self.insert('end', u'   ' + _('The "hold"-state for this package remains unchanged.') + u'\n\n')
        else:
            self.insert('end', u'   ' +_('This package will no longer be held back.') + u'\n')
            self.image_create('end', image=images[1], align='center')
            self.insert('end', u'   ' + _('This package will not be held back.') + u'\n\n')

        if HAVE_APTITUDE:
            self.insert('end', _('Packages that are currently held back') + u' ' +\
                    _('with both "apt-get" and "aptitude" are highlighted') + u' ')
            self.insert('end', _('orange'), 'orange')
            self.insert('end', u'.\n')

            self.insert('end', _('Packages that are currently held back') + u' ' +\
                   _('only with "apt-get" are highlighted') + u' ')
            self.insert('end', _('green'), 'green')
            self.insert('end', u'.\n')

            self.insert('end', _('Packages that are currently held back') + u' ' +\
                   _('only with "aptitude" are highlighted') + u' ')
            self.insert('end', _('pink'), 'pink')
            self.insert('end', u'.\n\n')
        else:
            self.insert('end', _('Packages that are currently held back') + u' ' +\
                        _('are highlighted') + u' ')
            self.insert('end', _('orange'), 'orange')
            self.insert('end', u'.\n\n')

        self.insert('end', _('The package list supports the following keyboard shortcuts:') + u'\n')
        self.insert('end', _('Ctrl+H : put selected package(s) on hold.') + u'\n')
        self.insert('end', _('Ctrl+U : "un-hold" selected package(s).') + u'\n')
        self.insert('end', _('Space : toggle the "hold"-state for selected package(s).') + u'\n')
        self.configure(state='disabled')

###############################################################################

class HoldsConfigDialog(BaseDialog):
    def __init__(self, master=None, **kw):
        BaseDialog.__init__(self, master,
                buttons=(_('Apply'), _('Close')), icon='preferences', **kw)
        self.protocol('WM_DELETE_WINDOW', self.hide)

        self.is_busy = 0
        self.apt_holds = [] # list of current apt-holds
        self.aptitude_holds = [] # list of current aptitude-holds
        self.installed = [] # list of installed pkgs
        # dictionaries that reflect the "hold"-state of the installed packages
        # items look like 'pkgname':'state', where state is one of
        # "hold_apt" or "hold_none", plus, if aptitude is installed,
        # "hold_aptitude" and "hold_both"
        self.pkg_state = {}
        # same as above, with an additional possible state "unhold";
        # this reflects the state of the pkgs with selected changes considered
        self.pkg_state_edited = {}

        self.img_info = Icon('dialog_info')
        self.img_error = Icon('dialog_error')
        self.img_filter = Icon('filter')
        # checkbox icons:
        self.img_checked = Tkinter.PhotoImage(
            data=('R0lGODlhDQANABEAACwAAAAADQANAIEAAAB/f3/f39////8CJ4yPNgHtL'
                  'xYYtNbIbJ146jZ0gzeCIuhQ53NJVNpmryZqsYDnemT3BQA7'))
        self.img_unchecked = Tkinter.PhotoImage(
            data=('R0lGODlhDQANABEAACwAAAAADQANAIEAAAB/f3/f39////8CIYyPNgHtL'
                  'xYYtNbIrMZTX+l9WThwZAmSppqGmADHcnRaBQA7'))
        self.img_unhold = Tkinter.PhotoImage(
            data=('R0lGODlhDQANAMIAAAAAAK8FBX9/f9/f3////wAAAAAAAAAAACH+EUNyZ'
                  'WF0ZWQgd2l0aCBHSU1QACwAAAAADQANAAADLyi6rCQgyjkeIDjnCkkI2U'
                  'dw1weao4WZYtqtrXt5LEbSp3jHuqppnIFwSKz8jpgEADs='))

        self.message.grid_remove()
        self.detail.grid_remove()
        st = InfoText(self.head,
                images=(self.img_checked, self.img_unchecked, self.img_unhold),
                width=5, height=7)
        st.grid(row=0, column=1, rowspan=2, padx=5, pady=10, sticky='news')

        self.frame.configure(relief='groove', bd=2)
        self.frame.grid_configure(padx=5, pady=5)
        self.frame.grid_rowconfigure(1, weight=1)
        self.frame.grid_columnconfigure(1, weight=1)

        scrolledtree = ScrolledTreeview(self.frame)
        scrolledtree.grid(row=1, column=0, columnspan=3, sticky='news',
                            pady=5, padx=10)
        self.pkglist = scrolledtree.treeview
        self.pkglist.configure(show=('tree', ), columns=('installed',))
        self.pkglist.column('#0', width=35, stretch=0)

        if HAVE_APTITUDE:
            self.pkglist.tag_configure('hold_both', background='#FF903D')
            self.pkglist.tag_configure('hold_apt', background='#2CF88A')
            self.pkglist.tag_configure('hold_aptitude', background='#FA89DD')
        else:
            self.pkglist.tag_configure('hold_apt', background='#FF903D')
        self.pkglist.tag_configure('checked', image=self.img_checked)
        self.pkglist.tag_configure('unchecked', image=self.img_unchecked)
        self.pkglist.tag_configure('unhold', image=self.img_unhold)

        self.filter = Tkinter.StringVar(value='*')
        self.holdsonly = Tkinter.IntVar(value=0)
        self.usefilter = Tkinter.IntVar(value=0)

        Tkinter.Label(self.frame, text=_('Filter:'), anchor='w').grid(
                            row=2, column=0, pady=10, padx=10, sticky='w')
        entry = ttk.Entry(self.frame, textvariable=self.filter)
        entry.grid(row=2, column=1, pady=10, padx=(10,0), sticky='ew')
        entry.bind('<Return>', self.enable_filter)

        Tkinter.Checkbutton(self.frame, image=self.img_filter, takefocus=1,
                    command=self.toggle_filter, bd=1,
                    indicatoron=0, onvalue=1, offvalue=0,
                    variable=self.usefilter).grid(
                    row=2, column=2, padx=(2, 25))

        ttk.Checkbutton(self.frame, variable=self.holdsonly,
                    text=_('Show only held back and selected packages'),
                    command=self.toggle_filter).grid(row=3, column=1,
                    columnspan=2, padx=(10, 25), pady=(0,10), sticky='w')

        if not HAVE_PERMISSIONS:
            msg = _('No write access to') + u' ' +\
                fsdecode('/var/lib/dpkg/')+ u' .\n' +\
                _('You may view the settings but will not be able to apply changes.')
            showinfo(title='Update Notifier Applet',
                message=_('Please note:'), detail=msg)

        self.pkglist.bind('<ButtonPress-1>', self.toggle_clicked)
        self.pkglist.bind('<Control-h>', self.hold_selected)
        self.pkglist.bind('<Control-u>', self.unhold_selected)
        self.pkglist.bind('<space>', self.toggle_selected)
        self.pkglist.bind('<FocusOut>', self._on_focus_out)

        self.buttons()[0].configure(command=self.apply, state='disabled')
        self.buttons()[1].configure(command=self.hide)
        self.assign_hotkeys()

    ############################################################

    def _trace_changes(self):
        self.update_idletasks()
        changed = self.pkg_state != self.pkg_state_edited
        if changed and HAVE_PERMISSIONS:
            self.buttons()[0].configure(state='normal')
        else:
            self.buttons()[0].configure(state='disabled')
        return changed

    ############# PKGLIST METHODS ##############################

    # ttk.Treeview is broken when used with wantobjects=0 ,
    # so we have to add workarounds for item('tags') and selection()
    def get_tags(self, pkgname):
        return(self.tk.splitlist(
                    self.tk.call(self.pkglist._w, 'item', pkgname, '-tags')))

    def selection_get(self):
        return(self.tk.splitlist(self.tk.call(self.pkglist._w, 'selection')))

    def _on_focus_out(self, event):
        # remove selection when focus leaves the list
        self.pkglist.selection_set(())

    def _toggle_pkg(self, p):
        # helper function for the toggle_... methods
        # toggle the checkbox for package "p"
        tags = list(self.get_tags(p))
        if 'unchecked' in tags:
            tags.remove('unchecked')
            tags.append('checked')
            if HAVE_APTITUDE:
                self.pkg_state_edited[p] = 'hold_both'
            else:
                self.pkg_state_edited[p] = 'hold_apt'
        # handle third checkbox state for pkgs only held for apt *or* aptitude
        elif 'unhold' in tags:
            tags.remove('unhold')
            if HAVE_APTITUDE and \
                    (p in self.apt_holds) ^ (p in self.aptitude_holds):
                tags.append('unchecked')
            else:
                tags.append('checked')
            self.pkg_state_edited[p] = self.pkg_state[p]
        else:
            tags.remove('checked')
            if (p in self.apt_holds) or (p in self.aptitude_holds):
                tags.append('unhold')
                self.pkg_state_edited[p] = 'unhold'
            else:
                tags.append('unchecked')
                self.pkg_state_edited[p] = 'hold_none'
        self.pkglist.item(p, tags=tags)

    def hold_selected(self, *event):
        while self.is_busy:
            pass
        self.is_busy = 1
        for p in self.selection_get():
            if p in self.apt_holds and p in self.aptitude_holds:
                self.pkglist.item(p, tags=('hold_both', 'checked'))
            elif p in self.apt_holds:
                self.pkglist.item(p, tags=('hold_apt', 'checked'))
            elif p in self.aptitude_holds:
                self.pkglist.item(p, tags=('hold_aptitude', 'checked'))
            else:
                self.pkglist.item(p, tags=('checked',))
            if HAVE_APTITUDE:
                self.pkg_state_edited[p] = 'hold_both'
            else:
                self.pkg_state_edited[p] = 'hold_apt'
        self._trace_changes()
        self.is_busy = 0

    def unhold_selected(self, *event):
        while self.is_busy:
            pass
        self.is_busy = 1
        for p in self.selection_get():
            if p in self.apt_holds and p in self.aptitude_holds:
                self.pkglist.item(p, tags=('hold_both', 'unhold'))
            elif p in self.apt_holds:
                self.pkglist.item(p, tags=('hold_apt', 'unhold'))
            elif p in self.aptitude_holds:
                self.pkglist.item(p, tags=('hold_aptitude', 'unhold'))
            else:
                self.pkglist.item(p, tags=('unchecked',))
            self.pkg_state_edited[p] = 'hold_none'
        self._trace_changes()
        self.is_busy = 0

    def toggle_selected(self, *event):
        while self.is_busy:
            pass
        self.is_busy = 1
        for p in self.selection_get():
            self._toggle_pkg(p)
        self._trace_changes()
        self.is_busy = 0

    def toggle_clicked(self, event):
        while self.is_busy:
            pass
        self.is_busy = 1
        p = self.pkglist.identify_column(event.x)
        if p == '#0':
            p = self.pkglist.identify_row(event.y)
            if p:
                self._toggle_pkg(p)
        self._trace_changes()
        self.is_busy = 0

    ############################################################

    def apply_filter(self):
        while self.is_busy:
            pass
        self.is_busy = 1
        libmisc.tk_busy(self)
        for p in self.installed:
            self.pkglist.detach(p)
        f = self.filter.get().strip()
        if not f:
            f = '*'
        self.filter.set(f)
        matches = fnmatch.filter(self.installed, f)
        for pkgname in matches:
            if self.holdsonly.get():
                tags = self.get_tags(pkgname)
                if 'unchecked' in tags and not (('hold_apt' in tags) or \
                        ('hold_aptitude' in tags) or ('hold_both' in tags)):
                    continue
            self.pkglist.move(pkgname, '', 'end')
        libmisc.tk_busy_forget(self)
        self.is_busy = 0

    def enable_filter(self, event):
        if not self.usefilter.get():
            self.usefilter.set(1)
        self.apply_filter()

    def toggle_filter(self):
        if self.usefilter.get():
            self.apply_filter()
        else:
            filter = self.filter.get()
            self.filter.set('*')
            self.apply_filter()
            self.filter.set(filter)

    ############################################################

    def get_changed(self):
        # return a list of tuples like (pkgname, newstate)
        changed = []
        for p in self.pkg_state:
            if self.pkg_state[p] != self.pkg_state_edited[p]:
                changed.append(p)
        changed.sort()
        changed = [(p, self.pkg_state_edited[p]) for p in changed]
        return changed

    def get_holds(self, cmd='apt'):
        if cmd == 'apt':
            cmdline = 'apt-mark showhold'
        else:
            cmdline = 'aptitude -q=2 -F "%p" search ~ahold'
        # aptitude prints a warning if a lockfile is not available, so
        # check the output against self.installed to avoid adding the
        # warning to our list of holds
        return [x.strip() for x in getoutput(cmdline).splitlines()
                    if x.strip() in self.installed]

    def load_pkglist(self, firsttime=True):
        # query installed pkgs only once at program start
        if firsttime:
            self.installed = [x.strip() for x in
                                getoutput('apt-mark showauto').splitlines()]
            self.installed += [x.strip() for x in
                                getoutput('apt-mark showmanual').splitlines()]
            self.installed.sort()

        self.apt_holds = self.get_holds('apt')
        if HAVE_APTITUDE:
            self.aptitude_holds = self.get_holds('aptitude')

        self.pkg_state = {}
        for p in self.installed:
            h ='hold_none'
            if p in self.apt_holds and p in self.aptitude_holds:
                h = 'hold_both'
            elif p in self.apt_holds:
                h = 'hold_apt'
            elif p in self.aptitude_holds:
                h = 'hold_aptitude'
            self.pkg_state[p] = h
        self.pkg_state_edited = self.pkg_state.copy()

    def update_pkglist(self, firsttime=True):
        # called with firsttime=True at program start -> create items
        # later from apply() with firsttime=False -> only update item tags
        self.load_pkglist(firsttime=firsttime)
        for p in self.installed:
            tags = ('unchecked',)
            if HAVE_APTITUDE:
                if self.pkg_state[p] == 'hold_both':
                    tags = ('hold_both', 'checked')
                elif self.pkg_state[p] == 'hold_apt':
                    tags = ('hold_apt', 'unchecked')
                elif self.pkg_state[p] == 'hold_aptitude':
                    tags = ('hold_aptitude', 'unchecked')
            else:
                if self.pkg_state[p] == 'hold_apt':
                    tags = ('hold_apt', 'checked')
            if firsttime:
                self.pkglist.insert(parent='', index='end',
                                        iid=p, values=(p,), tags=tags)
            else:
                self.pkglist.item(p, tags=tags)
        self.pkglist.focus_set()
        if firsttime and self.installed:
            self.pkglist.focus(self.installed[0])
            self.pkglist.selection('set', self.installed[0])

    ############## BUTTON CALLBACKS ####################

    def ask_apply(self, changed, add_apt, remove_apt,
                        add_aptitude, remove_aptitude):
        # helper func for apply()
        msg = _('The following changes will be applied') + u':\n\n'
        for p, s in changed:
            if p in add_apt and p in add_aptitude:
                msg += _('"%s" will be held back') % p + u'\n'
            elif p in add_apt:
                if HAVE_APTITUDE:
                    msg += _('"%s" will be held back with apt-get') % p +\
                           u' ' + _('(already held back with aptitude)') + u'\n'
                else:
                    msg += _('"%s" will be held back') % p + u'\n'
            elif p in add_aptitude:
                msg += _('"%s" will be held back with aptitude') % p +\
                       u' (' + _('already held back with apt-get') + u')\n'
        msg += u'\n'
        for p, s in changed:
            if p in remove_apt and p in remove_aptitude:
                msg += _('"%s" will no longer be held back') % p + u'\n'
            elif p in remove_apt:
                if HAVE_APTITUDE:
                    msg += _('"%s" will no longer be held back with apt-get') % p +\
                           u' (' + _('already not held back with aptitude') + u')\n'
                else:
                    msg += _('"%s" will no longer be held back') % p + u'\n'
            elif p in remove_aptitude:
                msg += _('"%s" will no longer be held back with aptitude') % p +\
                       u' (' + _('already not held back with apt-get') + u')\n'

        dtl = _('In the overview below you can check again the list of packages') +\
              u' ' + _('whose "hold"-marks will be newly added or removed.')

        res = msgbox_askokcancel(parent=self, message=_('Apply changes now?'),
                                    detail=dtl, text=msg, default=1)
        return res

    def get_cmds(self, add_apt, remove_apt, add_aptitude, remove_aptitude):
        # helper func for apply()
        cmds = []
        if add_apt:
            cmds.append('apt-mark hold %s' % ' '.join([fsencode(x) for x in add_apt]))
        if remove_apt:
            cmds.append('apt-mark unhold %s' % ' '.join([fsencode(x) for x in remove_apt]))
        if add_aptitude:
            cmds.append('aptitude hold %s' % ' '.join([fsencode(x) for x in add_aptitude]))
        if remove_aptitude:
            cmds.append('aptitude unhold %s' % ' '.join([fsencode(x) for x in remove_aptitude]))
        return cmds

    def get_todo_lists(self, pkglist):
        # helper func for apply()
        add_apt, remove_apt, add_aptitude, remove_aptitude = [], [], [], []
        for p, s in pkglist:
            if s in ('hold_both', 'hold_apt'):
                if not p in self.apt_holds:
                    add_apt.append(p)
                if HAVE_APTITUDE and not p in self.aptitude_holds:
                    add_aptitude.append(p)
            else: # s == 'unhold'
                if p in self.apt_holds:
                    remove_apt.append(p)
                if HAVE_APTITUDE and p in self.aptitude_holds:
                    remove_aptitude.append(p)
        return (add_apt, remove_apt, add_aptitude, remove_aptitude)

    def apply(self, *event):
        if not HAVE_PERMISSIONS or self.is_busy or \
                        self.buttons()[0]['state'] == 'disabled':
            return
        self.is_busy = 1
        libmisc.tk_busy(self)

        changed = self.get_changed()
        if not changed:
            showinfo(parent=self, title='Update Notifier Applet',
                        message=_('Information'),
                        detail=_('No changes to the currently saved state.'))
            self.tk.call('tk', 'busy', 'forget', self._w)
            self.is_busy = 0
            return

        add_apt, remove_apt, add_aptitude, remove_aptitude = self.get_todo_lists(changed)

        if self.ask_apply(changed, add_apt, remove_apt,
                            add_aptitude, remove_aptitude):
            splash = Splash(self, message=_('Applying changes, please wait...'),
                            detail='', buttons=(_('Close'),),
                            text=_('Running command(s):') + u'\n')
            splash.buttons()[0].focus_set()
            splash.buttons()[0].configure(state='disabled')
            splash.show()

            cmds = self.get_cmds(add_apt, remove_apt, add_aptitude, remove_aptitude)

            i = 0
            n = len(cmds)
            err = 0
            while cmds:
                i += 1
                cmd = cmds[-1]
                cmds = cmds[:-1]
                splash.detail.configure(
                            text=_('Processing command') + u' %d / %d' % (i, n))

                splash.append_text(cmd + ' ...')
                splash.update_idletasks()
                sts, out = getstatusoutput('lsof /var/lib/dpkg/lock')# FIXME: this seems weak
                if not sts:
                    err = 1
                    detail = _('Could not get lock') + u' /var/lib/dpkg/lock.\n' +\
                             _('Please try again later.')
                    splash.append_text(u' %s\n' % _('error'))
                    break
                sts, out = getstatusoutput(cmd)
                if sts:
                    err = 1
                    detail = _('Error while processing command') + u':\n' +\
                             fsdecode(cmd) + u'\n' +\
                             _('The changes may have been applied partially.') + u'\n' +\
                             _('Please check again the list of held back packages.')
                    splash.append_text(u' %s\n' % _('error'))
                    break
                splash.append_text(u' %s\n' % _('ok'))

            splash.detail.configure(text=_('Updating packages list') + u'...')
            splash.update_idletasks()
            self.update_pkglist(firsttime=False)

            if err:
                splash.icon.configure(image=self.img_error)
                splash.message.configure(text=_('Error'))
                splash.detail.configure(text=detail)
            else:
                splash.icon.configure(image=self.img_info)
                splash.message.configure(text=_('Information'))
                splash.detail.configure(text=_('Changes successfully applied.'))
            splash.buttons()[0].configure(state='normal', command=splash.destroy)
            splash.assign_hotkey(splash.buttons()[0])
            self.wait_window(splash)
            self.buttons()[0].configure(state='disabled')
            self.update_idletasks()

        self.tk.call('tk', 'busy', 'forget', self._w)
        libmisc.tk_busy_forget(self)
        self.is_busy = 0

    def force_quit(self, *args):
        # used with signal handler
        self.quit()

    def hide(self, *args):
        if self.is_busy:
            self.bell()
            return
        self.quit()

###############################################################################

class Splash(BaseDialog):
    def __init__(self, master=None, message='', detail='',
                    text=None, buttons=(), icon='clock', **kw):
        BaseDialog.__init__(self, master, buttons=buttons, message=message,
                            detail=detail, icon=icon, **kw)
        self.protocol('WM_DELETE_WINDOW', self.dummy_func)
        self.parent = master

        wl = self.winfo_screenwidth() / 4
        if not text is None:
            self.text = ScrolledText(self.frame, width=80, height=12)
            self.text.pack(fill='both', expand=1, padx=10, pady=10)
            self.append_text(text)

            wl1 = self.text.winfo_reqwidth() - 120
            if wl1 > 150: # sanity check
                wl = wl1
        self.message.configure(wraplength=wl)
        self.detail.configure(wraplength=wl)
        self.update_idletasks()

    def append_text(self, text):
        self.text.configure(state='normal')
        self.text.insert('end', text)
        self.text.see('end')
        self.text.configure(state='disabled')

    def dummy_func(self):
        pass

    def show(self):
        BaseDialog.show(self)
        self.grab_set()
        try:
            if self.parent and self.parent.winfo_viewable():
                self.transient(self.parent)
        except:
            pass
        self.update_idletasks()

###############################################################################

def main():
    root = libmisc.mainwindow()

    app = HoldsConfigDialog(root)
    root.quit_command(app.force_quit)
    splash = Splash(app, message=_('Please wait...'),
                    detail=_('Generating the list of installed packages...'))

    app.is_busy = 1
    libmisc.tk_busy(app)
    app.show()
    splash.show()
    app.update_pkglist()
    app.is_busy = 0
    libmisc.tk_busy_forget(app)

    splash.destroy()
    root.mainloop()
    root.destroy()

if __name__ == '__main__':
    main()
